#######################################################################
#
# NCM component for lcginfo daemon
#
# Charles Loomis <charles.loomis@cern.ch>
# Version 0.0.4 : 06/12/04 08:33
#
#  ** Generated file : do not edit **
#
#######################################################################

package NCM::Component::lcginfo;

use strict;
use NCM::Component;
use vars qw(@ISA $EC);
@ISA = qw(NCM::Component);
$EC=LC::Exception::Context->new->will_store_all;
use NCM::Check;

use EDG::WP4::CCM::Element;

use File::Path;

local(*DTA);


##########################################################################
sub Configure($$@) {
##########################################################################
    
    my ($self, $config) = @_;

    # Define paths for convenience. 
    my $base = "/software/components/lcginfo";

    # Default location for LCG software.
    my $config_path =  $config->getValue("$base/configPath");
 
    # Ensure that the base LCG directory exists.
    mkpath($config_path) unless (-d $config_path);

    # Pull out the location of the configuration file RELATIVE
    # to the path!
    my $config_file = $config_path . '/' . 
                      $config->getValue("$base/configFile");

    # Create the contents of the configuration file.
    my $contents = $self->fillTemplate($config);

    # Fill the file with the contents. 
    if ( ! -e $config_file ) {
	
	# Configuration file doesn't exist yet.  Create it. 
	open CONF, ">$config_file";
	print CONF $contents;
	close (CONF);
	$self->log("$config_file created");
	
    } else {
	
	# Already exists. Make backup and create new file. 
	my $result = LC::Check::file( $config_file,
				      backup => ".old",
				      contents => $contents,
				      );
	$self->log("$config_file updated") if $result;
    }

    # Pull out the location of the configuration script.
    my $config_script = $config->getValue("$base/configScript");

    # Execute this script passing the location of the config. file.
    # This creates the static LDIF file.
    if (-x $config_script) {
	system("$config_script $config_file");
	if ($?) {
	    $self->error("error running $config_script: " . ($?>>8));
	    return 1;
	}
    } else {
	$self->error("$config_script doesn't exist or isn't executable");
	return 1;
    }

    # Done. 
    return 1; 
}


# Since templating doesn't exist in quattor yet, do this by hand.
sub fillTemplate {

    my ($self,$config) = @_;

    # Define paths for convenience. 
    my $base = "/software/components/lcginfo";

    # Save the date.
    my $date = localtime();

    my $contents = "#\n# Generated by ncm-lcginfo on $date\n#\n";

    # Default location for LCG software.
    my $config_path =  $config->getValue("$base/configPath");
 
    # Pull out the location of the ldif file RELATIVE
    # to the path!
    my $ldif_file = $config_path . '/' . 
                    $config->getValue("$base/ldifFile");

    # Get the script locations.
    my $generic_script = $config->getValue("$base/genericScript");
    my $wrapper_script = $config->getValue("$base/wrapperScript");

    # Add these to the output.
    $contents .= 
	"ldif_file=$ldif_file\n" . 
	"generic_script=$generic_script\n" . 
	"wrapper_script=$wrapper_script\n\n";

    # Get a list of all of the templates.
    my @templates;
    if ($config->elementExists("$base/templates")) {
        my @elements = $config->getElement("$base/templates")->getList();
	foreach (@elements) {
	    push @templates, $_->getValue();
	}
    }

    # Write a line for each of the templates.
    foreach (@templates) {
	$contents .= "template=$_\n";
    }
    $contents .= "\n";

    # Get a list of all of the providers.
    my @providers;
    if ($config->elementExists("$base/providers")) {
        my @elements = $config->getElement("$base/providers")->getList();
	foreach (@elements) {
	    push @providers, $_->getValue();
	}
    }

    # Write a line for each of the providers.
    foreach (@providers) {
	$contents .= "providers=$_\n";
    }
    $contents .= "\n";

    # Get a list of all of the dynamicScript resources.
    my @dynamic;
    if ($config->elementExists("$base/dynamicScripts")) {
        @dynamic = $config->getElement("$base/dynamicScripts")->getList();
    }

    # Write a line for each of the dynamic scripts.
    foreach (@dynamic) {
	my %hash = $_->getHash();
	my $path = $hash{path}->getValue();
	my $script = $hash{script}->getValue();
	my $args = $hash{args}->getValue();
	$contents .= "dynamic_script=$path/$script $args\n";
    }
    $contents .= "\n";

    # Get a list of all of the literal entries.
    my @entries;
    if ($config->elementExists("$base/entries")) {
        @entries = $config->getElement("$base/entries")->getList();
    }

    # Write out the information for each entry.
    foreach my $entry (@entries) {
	my %ehash = $entry->getHash();
	my $dn = $ehash{dn}->getValue();
	$contents .= "$dn\n";
	my %attrs = $ehash{attributes}->getHash();
	foreach my $key (sort keys %attrs) {
	    my @values = $attrs{$key}->getList();
	    foreach my $v (@values) {
		my $value = $v->getValue();
		$contents .= "$key: $value\n";
	    }
	}
	$contents .= "\n";
    }

    return $contents;
}

1;      # Required for PERL modules
